# generated by datamodel-codegen:
#   filename:  openapi.json
#   timestamp: 2022-10-16T17:59:39+00:00

from __future__ import annotations

from uuid import UUID

from pydantic import BaseModel, Field

from .model import VBaseModel
