# generated by datamodel-codegen:
#   filename:  openapi.json
#   timestamp: 2022-10-16T17:59:39+00:00

from __future__ import annotations

from pydantic import BaseModel, Extra, Field

from .model import VBaseModel


class StatsModel(VBaseModel):

    totalWebhooksDelivered: float = Field(
        ...,
        description="The total amount of webhooks delivered across all streams",
    )
    totalWebhooksFailed: float = Field(
        ...,
        description="The total amount of failed webhooks across all streams",
    )
    totalLogsProcessed: float = Field(
        ...,
        description="The total amount of logs processed across all streams, this includes failed webhooks",
    )
    totalTxsProcessed: float = Field(
        ...,
        description="The total amount of txs processed across all streams, this includes failed webhooks",
    )
    totalTxsInternalProcessed: float = Field(
        ...,
        description="The total amount of internal txs processed across all streams, this includes failed webhooks",
    )
